@@ -12,7 +12,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the 
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA  02111-1307, USA.
  */
@@ -26,6 +26,9 @@ struct socks4_h {
 	guint8 vn;
 	guint8 cd;
 	guint16 dport;
+#ifdef ENABLE_IPV6
+	guint8 dip6[16];
+#endif
 	guint32 dip;
 	guint8 userid;
 };
@@ -34,7 +37,10 @@ struct socks5_h {
 	guint8  vn;
 	guint8  cd;
 	guint8  rsv;
-	guint8  atyp;	
+	guint8  atyp;
+#ifdef ENABLE_IPV6
+	guint8 dip6[16];
+#endif
 	guint32 dip;
 	guint32 dport;
 };
