@@ -16,6 +16,9 @@
 #endif
 
 #define _ISOC99_SOURCE
+#if defined(__FreeBSD_kernel__) && defined(__GLIBC__)
+#define _BSD_SOURCE 1
+#endif
 #include <math.h>
 #include <string.h>
 #include <glib/gi18n.h>
@@ -33,14 +36,14 @@
 
 #ifdef __linux__
 #include <linux/cdrom.h>
-#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
+#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD_kernel__) || defined(__sun)
 #include <sys/cdio.h>
 #endif
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 #include <sys/cdrio.h>
 #endif
 
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 # define GST_CDROM_IOCTL_CDCAPABILITY_REQUEST  CDIOCCAPABILITY
 # define GST_CDROM_IOCTL_EJECT_REQUEST         CDIOCEJECT
 #elif defined(__NetBSD__) || defined(__OpenBSD__)
@@ -184,11 +187,15 @@ gst_cdrom_is_cdrom_device (GnomeCDRom * cdrom, const char *device,
 
     fd = open (device, O_RDONLY | O_NONBLOCK);
     if (fd >= 0) {
+#ifdef __sun
+      res = TRUE;
+#else
       if (ioctl (fd, GST_CDROM_IOCTL_CDCAPABILITY_REQUEST, 0) >= 0) {
         res = TRUE;
       } else {
         GST_DEBUG ("ioctl() failed: %s", g_strerror (errno));
       }
+#endif
 
       close (fd);
     } else {
@@ -568,7 +575,23 @@ gst_cdrom_eos_msg (GstCDRom * cdrom, GstMessage * msg, GstBus * bus)
 
   GST_DEBUG ("received EOS %s", (was_last) ? "(last track)" : "");
 
-  if (was_last) {
+  if (was_last && GNOME_CDROM (cdrom)->playmode == GNOME_CDROM_SINGLE_TRACK) {
+    GnomeCDRomMSF start_msf, end_msf;
+    start_msf.minute = 0;
+    start_msf.second = 0;
+    start_msf.frame = 0;
+
+    end_msf.minute = 0;
+    end_msf.second = 0;
+    end_msf.frame = 0;
+
+    gst_cdrom_play (GNOME_CDROM (cdrom),
+		    cdrom->priv->status.track,
+		    &start_msf,
+		    cdrom->priv->status.track,
+		    &end_msf,
+		    NULL);
+  } else if (was_last) {
     gst_cdrom_set_playbin_state_to_null (cdrom);
     cdrom->priv->status.audio = GNOME_CDROM_AUDIO_COMPLETE;
   } else {
@@ -844,7 +867,7 @@ gst_cdrom_update_cd (GnomeCDRom * gnome_cdrom)
 /*
 	GstCdparanoiaCDRom *lcd = GST_CDPARANOIA_CDROM (cdrom);
 	GstCdparanoiaCDRomPrivate *priv;
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 	struct ioc_read_toc_single_entry tocentry;
 #elif defined(__NetBSD__) || defined(__OpenBSD__)
 	struct ioc_read_toc_entry tocentries;
@@ -862,7 +885,7 @@ gst_cdrom_update_cd (GnomeCDRom * gnome_cdrom)
 		return;
 	}
 
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD_kernel__)
 	if (ioctl (cdrom->fd, CDIOREADTOCHEADER, priv->tochdr) < 0) {
 #else
 	if (ioctl (cdrom->fd, CDROMREADTOCHDR, priv->tochdr) < 0) {
@@ -873,7 +896,7 @@ gst_cdrom_update_cd (GnomeCDRom * gnome_cdrom)
 		return;
 	}
 
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD_kernel__)
 	priv->track0 = priv->tochdr->starting_track;
 	priv->track1 = priv->tochdr->ending_track;
 #else
@@ -886,8 +909,8 @@ gst_cdrom_update_cd (GnomeCDRom * gnome_cdrom)
 	priv->track_info =
 	    g_malloc ((priv->number_tracks +
 		       1) * sizeof (GstCdparanoiaCDRomTrackInfo));
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD_kernel__)
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 	for (i = 0, j = priv->track0; i < priv->number_tracks; i++, j++) {
 		tocentry.track = j;
 		tocentry.address_format = CD_MSF_FORMAT;
@@ -899,7 +922,7 @@ gst_cdrom_update_cd (GnomeCDRom * gnome_cdrom)
 		tocentries.address_format = CD_MSF_FORMAT;
 #endif
 
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 		if (ioctl (cdrom->fd, CDIOREADTOCENTRY, &tocentry) < 0) {
 #else
 		if (ioctl (cdrom->fd, CDIOREADTOCENTRYS, &tocentries) < 0) {
@@ -909,7 +932,7 @@ gst_cdrom_update_cd (GnomeCDRom * gnome_cdrom)
 		}
 
 		priv->track_info[i].track = j;
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 		priv->track_info[i].audio_track =
 		    tocentry.entry.control != CDROM_DATA_TRACK ? 1 : 0;
 		ASSIGN_MSF (priv->track_info[i].address,
@@ -936,8 +959,8 @@ gst_cdrom_update_cd (GnomeCDRom * gnome_cdrom)
 			    tocentry.cdte_addr.msf);
 #endif
 	}
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD_kernel__)
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 	tocentry.track = CDROM_LEADOUT;
 	tocentry.address_format = CD_MSF_FORMAT;
 	if (ioctl (cdrom->fd, CDIOREADTOCENTRY, &tocentry) < 0) {
@@ -950,7 +973,7 @@ gst_cdrom_update_cd (GnomeCDRom * gnome_cdrom)
 		gst_cdparanoia_cdrom_invalidate (lcd);
 		return;
 	}
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 	ASSIGN_MSF (priv->track_info[priv->number_tracks].address,
 		    tocentry.entry.addr.msf);
 #else
@@ -1001,6 +1024,12 @@ gst_cdrom_get_status (GnomeCDRom * gnome_cdrom, GnomeCDRomStatus ** status,
     if (!gst_cdrom_ensure_open (cdrom)) {
       *status = NULL;
       GST_DEBUG ("ensure_open failed");
+      if (error) {
+        /* error message is never shown to user, so no need to translate */
+        *error = g_error_new (GNOME_CDROM_ERROR,
+                              GNOME_CDROM_ERROR_IO,
+                              "Could not open CD.");
+      }
       return FALSE;
     }
   }
